@@ -152,6 +152,13 @@ class RabbitEventBehavior extends Behavior {
      */
     private $transaction = null;
 
+    /**
+     * Whether or not to send all model attributes on
+     * update and delete events.
+     * @var bool
+     */
+    public bool $sendModelAttributes = false;
+
     /**
      * {@inheritdoc}
      */
@@ -733,8 +740,11 @@ class RabbitEventBehavior extends Behavior {
                     $oldAttributes[$preppedAttribute] = $changedAttributes[$rawKey];
                 }
 
+                $table = $this->owner::tableName();
+
                 // check if the attribute is used in a constructed one
-                if (array_key_exists($rawKey, array_keys($this->constructedAttributes))) {
+                if (array_key_exists($rawKey, $this->constructedAttributes)) {
+                    Yii::info("Constructed key exists for {$rawKey} on table {$table}...", __METHOD__);
                     $constructed = $this->constructedAttributes[$rawKey];
                     if (is_array($constructed)) {
                         foreach ($constructed as $constructedAttr => $displayConstructedAttr) {
@@ -743,6 +753,10 @@ class RabbitEventBehavior extends Behavior {
                     } else {
                         $newAttributes[$constructed] = ArrayHelper::getValue($this->owner, $constructed);
                     }
+                } else {
+                    Yii::info("Constructed key does not exist for {$rawKey} on table {$table}...", __METHOD__);
+                    Yii::info("Constructed Attributes: " . json_encode($this->constructedAttributes), __METHOD__);
+                    Yii::info("Array Keys of Constructed: " . json_encode(array_keys($this->constructedAttributes)), __METHOD__);
                 }
             }
 
@@ -780,6 +794,10 @@ class RabbitEventBehavior extends Behavior {
                     'author' => $this->getAuthor(),
                 ];
 
+                if ($this->sendModelAttributes === true) {
+                    $message['entity']['modelAttributes'] = $preppedAttributes;
+                }
+
                 if ($this->extraData !== null) {
                     $message['entity']['extra'] = $this->prepareExtra();
                 }
@@ -847,6 +865,10 @@ class RabbitEventBehavior extends Behavior {
                 'author' => $this->getAuthor(),
             ];
 
+            if ($this->sendModelAttributes === true) {
+                $message['entity']['modelAttributes'] = $this->prepareAttributes();
+            }
+
             if ($this->extraData !== null) {
                 $message['entity']['extra'] = $this->prepareExtra();
             }
