@@ -2,8 +2,9 @@
 
 namespace onespace\tools\rabbit\helpers;
 
-trait CLIHelper {
+use stdClass;
 
+trait CLIHelper {
     public bool $silent = false;
 
     /*
@@ -17,7 +18,8 @@ trait CLIHelper {
     /**
      * Write to the screen a parsed string _WITHOUT_ a trailing line break.
      */
-    protected function write(string $string, Log $log = Log::NONE): void {
+    protected function write(string $string, Log $log = Log::NONE, string $key = __METHOD__): void {
+        $log->yiiLog($string, $key);
         if ($this->silent) {
             return;
         }
@@ -34,7 +36,8 @@ trait CLIHelper {
     /**
      * Write to the screen a parsed string _WITH_ a trailing line break.
      */
-    protected function writeln(string $string, Log $log = Log::NONE): void {
+    protected function writeln(string $string, Log $log = Log::NONE, string $key = __METHOD__): void {
+        $log->yiiLog($string, $key);
         if ($this->silent) {
             return;
         }
@@ -51,7 +54,8 @@ trait CLIHelper {
     /**
      * Write to the screen an object or array of data, formatted in a readable way.
      */
-    protected function writeData(array|object $data, Log $log = Log::NONE): void {
+    protected function writeData(array|object $data, Log $log = Log::NONE, string $key = __METHOD__): void {
+        $log->yiiLog($data, $key);
         if ($this->silent) {
             return;
         }
@@ -70,59 +74,59 @@ trait CLIHelper {
     /**
      * Write "true" or "false" to the screen based on a parsed bool value _WITHOUT_ a trailing line break.
      */
-    protected function writeBool(bool $bool, Log $log = Log::NONE): void {
+    protected function writeBool(bool $bool, Log $log = Log::NONE, string $key = __METHOD__): void {
         if ($bool) {
-            $this->write('true', $log);
+            $this->write('true', $log, $key);
         } else {
-            $this->write('false', $log);
+            $this->write('false', $log, $key);
         }
     }
 
     /**
      * Write "true" or "false" to the screen based on a parsed bool value _WITH_ a trailing line break.
      */
-    protected function writelnBool(bool $bool, Log $log = Log::NONE): void {
+    protected function writelnBool(bool $bool, Log $log = Log::NONE, string $key = __METHOD__): void {
         if ($bool) {
-            $this->writeln('true', $log);
+            $this->writeln('true', $log, $key);
         } else {
-            $this->writeln('false', $log);
+            $this->writeln('false', $log, $key);
         }
     }
 
     /**
      * Write a "✔" or "✘" to the screen based on a parsed bool value _WITHOUT_ a trailing line break.
      */
-    protected function writeCheck(bool $bool, Log $log = Log::NONE): void {
+    protected function writeCheck(bool $bool, Log $log = Log::NONE, string $key = __METHOD__): void {
         if ($bool) {
-            $this->write('✔', $log);
+            $this->write('✔', $log, $key);
         } else {
-            $this->write('✘', $log);
+            $this->write('✘', $log, $key);
         }
     }
 
     /**
      * Write a "✔" or "✘" to the screen based on a parsed bool value _WITH_ a trailing line break.
      */
-    protected function writelnCheck(bool $bool, Log $log = Log::NONE): void {
+    protected function writelnCheck(bool $bool, Log $log = Log::NONE, string $key = __METHOD__): void {
         if ($bool) {
-            $this->writeln('✔', $log);
+            $this->writeln('✔', $log, $key);
         } else {
-            $this->writeln('✘', $log);
+            $this->writeln('✘', $log, $key);
         }
     }
 
     /**
      * Write a date & time Timestamp _WITHOUT_ a trailing line break.
      */
-    protected function writeTs(string $format = 'Y-m-d H:i:s', Log $log = Log::NONE): void {
-        $this->write(date($format), $log);
+    protected function writeTs(string $format = 'Y-m-d H:i:s', Log $log = Log::NONE, string $key = __METHOD__): void {
+        $this->write(date($format), $log, $key);
     }
 
     /**
      * Write a date & time Timestamp _WITH_ a trailing line break.
      */
-    protected function writelnTs(string $format = 'Y-m-d H:i:s', Log $log = Log::NONE): void {
-        $this->writeln(date($format), $log);
+    protected function writelnTs(string $format = 'Y-m-d H:i:s', Log $log = Log::NONE, string $key = __METHOD__): void {
+        $this->writeln(date($format), $log, $key);
     }
 
     /**
@@ -133,7 +137,7 @@ trait CLIHelper {
      *                                  the beginning and end of the line, for example `|`.
      *                                  Set to null to skip. Default: `null`
      */
-    protected function drawDotLine(int $count = 22, ?string $wrapper = null, Log $log = Log::NONE): void {
+    protected function drawDotLine(int $count = 22, ?string $wrapper = null, Log $log = Log::NONE, string $key = __METHOD__): void {
         $txt = "";
         if ($wrapper !== null) {
             $txt .= $wrapper;
@@ -142,7 +146,7 @@ trait CLIHelper {
         if ($wrapper !== null) {
             $txt .= $wrapper;
         }
-        $this->writeln($txt, $log);
+        $this->writeln($txt, $log, $key);
     }
 
     /**
@@ -153,7 +157,7 @@ trait CLIHelper {
      *                                  the beginning and end of the line, for example `|`.
      *                                  Set to null to skip. Default: `null`
      */
-    protected function drawDashLine(int $count = 22, ?string $wrapper = null, Log $log = Log::NONE): void {
+    protected function drawDashLine(int $count = 22, ?string $wrapper = null, Log $log = Log::NONE, string $key = __METHOD__): void {
         $txt = "";
         if ($wrapper !== null) {
             $txt .= $wrapper;
@@ -162,7 +166,7 @@ trait CLIHelper {
         if ($wrapper !== null) {
             $txt .= $wrapper;
         }
-        $this->writeln($txt, $log);
+        $this->writeln($txt, $log, $key);
     }
 
     /**
@@ -173,7 +177,7 @@ trait CLIHelper {
      *                                  the beginning and end of the line, for example `|`.
      *                                  Set to null to skip. Default: `null`
      */
-    protected function drawPlusLine(int $count = 22, ?string $wrapper = null, Log $log = Log::NONE): void {
+    protected function drawPlusLine(int $count = 22, ?string $wrapper = null, Log $log = Log::NONE, string $key = __METHOD__): void {
         $txt = "";
         if ($wrapper !== null) {
             $txt .= $wrapper;
@@ -182,7 +186,7 @@ trait CLIHelper {
         if ($wrapper !== null) {
             $txt .= $wrapper;
         }
-        $this->writeln($txt, $log);
+        $this->writeln($txt, $log, $key);
     }
 
     /**
@@ -193,7 +197,7 @@ trait CLIHelper {
      *                                  the beginning and end of the line, for example `|`.
      *                                  Set to null to skip. Default: `null`
      */
-    protected function drawStarLine(int $count = 22, ?string $wrapper = null, Log $log = Log::NONE): void {
+    protected function drawStarLine(int $count = 22, ?string $wrapper = null, Log $log = Log::NONE, string $key = __METHOD__): void {
         $txt = "";
         if ($wrapper !== null) {
             $txt .= $wrapper;
@@ -202,7 +206,7 @@ trait CLIHelper {
         if ($wrapper !== null) {
             $txt .= $wrapper;
         }
-        $this->writeln($txt, $log);
+        $this->writeln($txt, $log, $key);
     }
 
     /**
@@ -213,7 +217,7 @@ trait CLIHelper {
      *                                  the beginning and end of the line, for example `|`.
      *                                  Set to null to skip. Default: `null`
      */
-    protected function drawUnderscoreLine(int $count = 22, ?string $wrapper = null, Log $log = Log::NONE): void {
+    protected function drawUnderscoreLine(int $count = 22, ?string $wrapper = null, Log $log = Log::NONE, string $key = __METHOD__): void {
         $txt = "";
         if ($wrapper !== null) {
             $txt .= $wrapper;
@@ -222,7 +226,7 @@ trait CLIHelper {
         if ($wrapper !== null) {
             $txt .= $wrapper;
         }
-        $this->writeln($txt, $log);
+        $this->writeln($txt, $log, $key);
     }
 
     /**
@@ -233,7 +237,7 @@ trait CLIHelper {
      *                                  the beginning and end of the line, for example `|`.
      *                                  Set to null to skip. Default: `null`
      */
-    protected function drawHashLine(int $count = 22, ?string $wrapper = null, Log $log = Log::NONE): void {
+    protected function drawHashLine(int $count = 22, ?string $wrapper = null, Log $log = Log::NONE, string $key = __METHOD__): void {
         $txt = "";
         if ($wrapper !== null) {
             $txt .= $wrapper;
@@ -242,7 +246,7 @@ trait CLIHelper {
         if ($wrapper !== null) {
             $txt .= $wrapper;
         }
-        $this->writeln($txt, $log);
+        $this->writeln($txt, $log, $key);
     }
 
     /*
@@ -275,7 +279,7 @@ trait CLIHelper {
     /**
      * Draw out a table on the CLI of parsed headings and values.
      */
-    protected function table(array $headings, array $values, Log $log = Log::NONE): void {
+    protected function table(array $headings, array $values, Log $log = Log::NONE, string $key = __METHOD__): void {
         $maxLen = [];
         foreach ($headings as $heading) {
             $maxLen[$heading] = strlen($heading);
@@ -324,7 +328,7 @@ trait CLIHelper {
         if (!$multi) {
             $str .= "|\n";
         }
-        $this->writeln($str, $log);
+        $this->writeln($str, $log, $key);
     }
 
     /*
@@ -338,9 +342,23 @@ trait CLIHelper {
     /**
      * In debug mode, print the data to the screen.
      */
-    protected function printProtoFile(\Google\Protobuf\Internal\Message $bin, Log $log = Log::DEBUG): void {
-        $this->writeln($bin->serializeToJsonString() . PHP_EOL, $log);
-        $this->writeln(bin2hex($bin->serializeToString()) . PHP_EOL, $log);
+    protected function printProtoFile(\Google\Protobuf\Internal\Message $bin, Log $log = Log::DEBUG, $key = __METHOD__): void {
+        $this->writeln($bin->serializeToJsonString() . PHP_EOL, $log, $key);
+        $this->writeln(bin2hex($bin->serializeToString()) . PHP_EOL, $log, $key);
+    }
+
+    /**
+     * Assumes we're in a service handler and $this->body exists. This prints out `$this->body`
+     */
+    protected function writeBody(Log $log = Log::NONE, string $key = __METHOD__): void {
+        if (!isset($this->body)) {
+            return;
+        }
+        if (gettype($this->body) === "array" || $this->body instanceof stdClass) {
+            $this->writeData($this->body, $log, $key);
+        } else {
+            $this->writeData(json_decode($this->body->rawPacket->serializeToJsonString()), $log, $key);
+        }
     }
 
     /*
